function precise=local_optim_ds(hol,found,lambda,pixel,w1,FOV_size1,FOV_size2,N,tt,factors,ranges,MEXmodelFAST)
%-------------------------------------------------------------------------%
% Programmed by: Mozhdeh Seifi - Laboratoire Hubert Curien 
%                               http://laboratoirehubertcurien.fr/
% 
% Contact : corinne.fournier@univ_st_etienne.fr
% Sept 2011
%-------------------------------------------------------------------------%
%-------------------------------------------------------------------------%
% Goal : Data fitting using the start point found by global detection
%
%-------------------------------------------------------------------------%
% License owner: Mozhdeh Seifi, Corinne Fournier, Loic Denis
%
% This work is licensed under a Creative Commons Attribution 
% Non-Commercial No Derivatives 3.0 France license. In essence, 
% you are free to copy and communicate the work in its current form
% for non-commercial purposes, as long as you attribute the authors
% and abide by the license terms.
% You may not alter or adapt the work in any way.
%
% To view a copy of this license, visit
% http://creativecommons.org/licenses/by-nc-nd/3.0/
%-------------------------------------------------------------------------%
%
% inputs: 
% hol           = hologram
% found         = coordinates of found particles roughly found by global detection , [x(in pixel),y(in pixel),z(in meter),r(in meter)]
% lambda        = wavelength in meter
% pixel         = pixel size in meter
% w1            = weighting matrix or Mask
% FOV_size1 
% & FOV_size2   = indexes of pixels in the hologram which contain the original hologram's data
% tt            = Max{ K(fill-factor) or N(down_sampling)-1 } for model calculation
% N             = N(down_sampling) for calculation of bounds on parameters _ min=1
% factors       = [dx dz dr corf itter] options of optimization, 
%               dx=minimum change on parameter x in meters , dz=minimum change on parameter z in meters,
%               dr=minimum change on parameter r in meters,coef is multiplied to CRLB to stop optimization,
%               itter=maximum iteration number
% ranges        = [Zmin Zmax;Rmin Rmax] in meters
% MEXmodelFAST  = 1 if model with multi-threading is required
%-------------------------------------------------------------------------%
% outputs:
% precise       = x(in pixel),y(in pixel),z(in meter),r(in meter) coordinates of particles after local optimization
%
%-------------------------------------------------------------------------%
%
%-------------------------------------------------------------------------%



map_size=size(hol);

if (factors(1)~=0)
    factorR=factors(3)/factors(1);
    omega=factors(2)/factors(1);
else
     factorR=0.5660;
end

[n l]=size(found);        % found comes from global detection. It is n*4 in the case of n particles
                          % - [x y z r]

 ws=sum(sum(w1(FOV_size1,FOV_size2)));

for i=1:n % n is the number of particles, Here it is equal to 1
%      omega=pixel*map_size(1)/2/found(i,3)/1.5;
     found_c=found(i,1:4);
    if (factors(1)==0) 
        omega=pixel*map_size(1)/2/found(i,3)/1.5;
    end

     found_c(3)=found_c(3)/omega;
     found_c(4)=found_c(4)/factorR;

     options = optimset('Jacobian','off','TolX',factors(4)*factors(1),'TolFun',1e-40,'MaxIter',factors(5),'Display','iter','MaxFunEvals',10^50);%optimset('OutputFcn', @outfun_nested,'Jacobian','off','TolX',2*dx,'TolFun',1e-40,'MaxIter',50,'Display','iter','MaxFunEvals',10^50);
     
     %local optimization
     precise(i,1:4)= lsqnonlin(@difference,found_c,[-map_size(1)/2*pixel*N -map_size(2)/2*pixel*N ranges(1,1)/omega ranges(2,1)/factorR],[(map_size(1)/2-1)*pixel*N (map_size(2)/2-1)*pixel*N ranges(1,2)/omega ranges(2,2)/factorR],options);   
 
     precise(i,3)=precise(i,3)*omega;
%      precise(i,1:2)=precise(i,1:2)/pixel;
     precise(i,4)=precise(i,4)*factorR;

 

end


%% 
    function [R]=difference(found_k) % input is the found particle. Outputs are the cost function and the jacobian
    %
    % inputs:
    % found_k       = coordinates of particle during the current step of optimization
    %
    % outputs:
    % R             = vector form of each residu before being squared and summed up to form the cost function
    % JJ            = vector form of the jacobian
       
   hol(FOV_size1,FOV_size2) = hol(FOV_size1,FOV_size2)-sum(sum(hol(FOV_size1,FOV_size2).*w1(FOV_size1,FOV_size2)))/ws;


    
    
    % produce model -
    if (MEXmodelFAST==1)
        g_current=model_calc_faster(found_k(1,1),found_k(1,2),found_k(1,3)*omega,found_k(1,4)*factorR,pixel,lambda,map_size,0,tt);
    else
        g_current=model_mex(found_k(1,1),found_k(1,2),found_k(1,3)*omega,found_k(1,4)*factorR,pixel,lambda,map_size,0,tt);
    end
    wg=sum(sum(w1.*g_current)); % wg
    wrg=sum(sum(w1.*hol.*g_current)); %wrg
    wg2=sum(sum(w1.*(g_current.^2)));%wg2
%    ws=sum(sum(w1));
    wr=sum(sum(w1.*hol));
    alphall=ws.*(wg.*wr-wrg)/(ws.*wg2 - wg.^2);
    I_p=(wg.^2.*wr-wg.*wrg)./(ws.*wg2 - wg.^2);

    
    % creating the cost function
    pp=zeros(map_size(1),map_size(2));
%     I00=I0+I_p;
    pp(FOV_size1,FOV_size2) = hol(FOV_size1,FOV_size2) - ( I_p - alphall.* g_current(FOV_size1,FOV_size2));%+I00*(alphall/2/I00).^2.*( (found_k(1,4)*factorR)^2 *pi/lambda/ (found_k(1,3)*omega) )^2*(jini(FOV_size1,FOV_size2).^2)) ;
     % to have bigger difference between cost function values
    R=reshape(pp,[map_size(1)*map_size(2) 1]);
    

    end
end
