 %-------------------------------------------------------------------------%
% Programmed by: Mozhdeh Seifi - Laboratoire Hubert Curien 
%                               http://laboratoirehubertcurien.fr/
%
% Contact : corinne.fournier@univ_st_etienne.fr
% Sept 2011
%-------------------------------------------------------------------------%
%
% Goal : Computing the DH reconstruction theoretical accuracy (deltax,deltaz,deltar) in the sensor center of a spherical particle (parameters (z,r).
% This algorithm uses numerical computation of CRLB for a spherical
% particle.
%-------------------------------------------------------------------------%
% License owner: Mozhdeh Seifi, Corinne Fournier, Loic Denis
%
% This work is licensed under a Creative Commons Attribution 
% Non-Commercial No Derivatives 3.0 France license. In essence, 
% you are free to copy and communicate the work in its current form
% for non-commercial purposes, as long as you attribute the authors
% and abide by the license terms.
% You may not alter or adapt the work in any way.
%
% To view a copy of this license, visit
% http://creativecommons.org/licenses/by-nc-nd/3.0/
%-------------------------------------------------------------------------%
% inputs:  
% x : vertical coordinate of the particle on sensor (unit meters)
% y : horizontal coordinate of the particle on sensor (unit meters)
% z : depth coordinate of the object (unit meters)
% r : radius of the object (unit meters)
% pix : pixel size (unit meters)
% lambda : wavelenght (unit meters)
% sizeH : row vector of 2 columns which defines the hologram's size in vertical and horizontal axes respectively
% sigm : the standard deviation of the noise (assume to be white and gaussian)
% n : (downsampling) ? (downsampling rate in pixels : fill-factor) 
% A is the peak amplitude (half of the Peak to peak amplitude) 
% if A =0 , signal aplitude is considered to be =pi*r*r/2/lambda/z 
% if A~=0 , the A value is taken into account 
%-------------------------------------------------------------------------%
% outputs: 
% deltax,deltay,deltaz,deltar : theoretical accuracy in the center of the sensor (unit meters)
%-------------------------------------------------------------------------%
% Test: 
% pix=6.7e-6;z=0.25;r=45e-6;lambda=0.532e-6;sizeH=[1024,1024];sigm=0.05;A=10;
% n=sqrt(0.7);x=0;y=0;
% [deltax,deltay,deltaz,deltar]=CRLB_point(x,y,z,r,pix,lambda,sizeH,sigm,n,A);
%!-------------------------------------------------------------------------%
%! processing time : ~0.395 s on Cetautomatix .
%-------------------------------------------------------------------------%
% [1] C. Fournier, L. Denis, et T. Fourner, "On the single point resolution of on-axis digital holography",
% Journal of the Optical Society of America A, vol. 27, no. 8, p.
% 1856-1862, 2010.
%-------------------------------------------------------------------------%
%*/